/* FILE:        BFQtLgplDialog.cpp
 * DATE:        1/7/2016
 * AUTHOR:      Jeremy Greene
 * COMPANY:     BitFlow, Inc.
 * COPYRIGHT:   Copyright (C) 2016, BitFlow, Inc.
 * DESCRIPTION: Implementation of BFQtLgplDialog.
 */

#include "BFQtLgplDialog_Private.h"

#include <QFile>

#include <atomic>

#if defined(_WIN32)
#   include <boost/thread.hpp>
namespace bf_this_thread = boost::this_thread;
#elif defined(__GNUC__)
#   include <thread>
namespace bf_this_thread = std::this_thread;
#else
#   error Thread support unknown for this platform.
#endif

enum class QrcInit
{
    None,
    InProgress,
    Complete
};

static std::atomic<QrcInit> s_loadedQrc (QrcInit::None);

static void loadQrc (void)
{
    QrcInit expectedLoadState = QrcInit::None;
    if (s_loadedQrc.compare_exchange_strong(expectedLoadState, QrcInit::InProgress))
    {
        Q_INIT_RESOURCE(BFQtLgplResources);
        s_loadedQrc.store(QrcInit::Complete);
    }
    else
    {
        while (s_loadedQrc.load() == QrcInit::InProgress)
            bf_this_thread::yield();
    }
}

BFQtLgplDialog::BFQtLgplDialog (QWidget *const parent)
    : QDialog (parent)
    , m_pd (*new PrivateData)
{
    loadQrc();

    m_pd.m_qtLgplLicensingDialog.setupUi(this);

    QFile lgplV2File (":/licenses/LGPLv21.txt");
    lgplV2File.open(QIODevice::ReadOnly);
    m_pd.m_qtLgplLicensingDialog.lgplV21TextBrowser->setText(lgplV2File.readAll());
    
    if (QT_VERSION >= QT_VERSION_CHECK(5, 4, 0))
    {
        QFile lgplV3File (":/licenses/LGPLv3.txt");
        lgplV3File.open(QIODevice::ReadOnly);
        m_pd.m_qtLgplLicensingDialog.lgplV3TextBrowser->setText(lgplV3File.readAll());
    }
    else
        m_pd.m_qtLgplLicensingDialog.lgplLicensesTabWidget->removeTab(m_pd.m_qtLgplLicensingDialog.lgplLicensesTabWidget->indexOf(m_pd.m_qtLgplLicensingDialog.lgplV3Widget));

    QFile qtLgplExceptionFile (":/licenses/LGPL_EXCEPTION.txt");
    qtLgplExceptionFile.open(QIODevice::ReadOnly);
    m_pd.m_qtLgplLicensingDialog.qtLgplExceptionTextBrowser->setText(qtLgplExceptionFile.readAll());

    // BitFlow only provides precompiled binaries on Windows. Clients are responsible
    // for Qt license compliance, on any other platform.
#if defined(_WIN32)
    m_pd.m_qtLgplLicensingDialog.linksLabel->setText("<p>For Qt sourcecode download, visit <a href=\"http://www.bitflow.com/support/opensource/\"><span style=\"text-decoration:underline;color:#0000ff;\">bitflow.com/support/opensource</span></a></p>");
#else
    m_pd.m_qtLgplLicensingDialog.linksLabel->setText("<p>For Qt sourcecode, contact the application vendor or your system administrator</p>");
#endif
}

BFQtLgplDialog::~BFQtLgplDialog (void)
{
    delete &m_pd;
}
