/* FILE:        AbstractAcquisitionInterface.cpp
 * DATE:        1/7/2016
 * AUTHOR:      Jeremy Greene
 * COMPANY:     BitFlow, Inc.
 * COPYRIGHT:   Copyright (C) 2016, BitFlow, Inc.
 * DESCRIPTION: Implementation of AbstractAcquisitionInterface.
 */

#include "AbstractAcquisitionInterface_Private.h"

#include "ImageViewStringHelpers.h"

ImageView::AbstractAcquisitionInterface::AbstractAcquisitionInterface (void)
    : m_pd (*new PrivateData(*this))
{ }

ImageView::AbstractAcquisitionInterface::~AbstractAcquisitionInterface (void)
{
    delete &m_pd;
}

// Method executed when the user interface is about to close.
void ImageView::AbstractAcquisitionInterface::uxClosed (void)
{ }

// Get the SignalEmitter.
ImageView::AcquisitionSignalHandler* ImageView::AbstractAcquisitionInterface::signalHandler (void)
{
    return &m_pd.m_signalHandler;
}
const ImageView::AcquisitionSignalHandler* ImageView::AbstractAcquisitionInterface::signalHandler (void) const
{
    return &m_pd.m_signalHandler;
}

// Update the GUI for all features that the interface has.
void ImageView::AbstractAcquisitionInterface::signalFeatureSetChange (void)
{
    m_pd.m_signalHandler.emitFeatureSetChanged();
}

// Update the GUI to reflect the current acquisition state.
void ImageView::AbstractAcquisitionInterface::signalAcquisitionStateChange (void)
{
    m_pd.m_signalHandler.emitAcquisitionStateChanged();
}

// Initiate a call to getNextFrame as soon as possible.
void ImageView::AbstractAcquisitionInterface::signalGetFrameNow (void)
{
    m_pd.m_signalHandler.emitGetFrameNow();
}

// Returns true/false, whether or not the interface supports acquisition setup.
bool ImageView::AbstractAcquisitionInterface::hasSetup (void) const
{
    return m_pd.m_hasSetup;
}
void ImageView::AbstractAcquisitionInterface::setHasSetup (const bool enabled)
{
    m_pd.m_hasSetup = enabled;
}

// Returns true/false, whether or not the interface supports acquisition setup with a mode.
bool ImageView::AbstractAcquisitionInterface::hasSetupMode (void) const
{
    return m_pd.m_hasSetupMode;
}
void ImageView::AbstractAcquisitionInterface::setHasSetupMode (const bool enabled)
{
    m_pd.m_hasSetupMode = enabled;
}

// Returns true/false, whether or not the interface supports acquisition cleanup.
bool ImageView::AbstractAcquisitionInterface::hasCleanup (void) const
{
    return m_pd.m_hasCleanup;
}
void ImageView::AbstractAcquisitionInterface::setHasCleanup (const bool enabled)
{
    m_pd.m_hasCleanup = enabled;
}

// Returns true/false, whether or not the interface supports acquisition triggering.
bool ImageView::AbstractAcquisitionInterface::hasTrigger (void) const
{
    return m_pd.m_hasTrigger;
}
void ImageView::AbstractAcquisitionInterface::setHasTrigger (const bool enabled)
{
    m_pd.m_hasTrigger = enabled;
}

// Returns true/false, whether or not the interface supports reverse acquisition start of a single frame.
bool ImageView::AbstractAcquisitionInterface::hasStartSingleReverse (void) const
{
    return m_pd.m_hasStartSingleReverse;
}
void ImageView::AbstractAcquisitionInterface::setHasStartSingleReverse (const bool enabled)
{
    m_pd.m_hasStartSingleReverse = enabled;
}

// Returns true/false, whether or not the interface supports reverse acquisition start.
bool ImageView::AbstractAcquisitionInterface::hasStartReverse (void) const
{
    return m_pd.m_hasStartReverse;
}
void ImageView::AbstractAcquisitionInterface::setHasStartReverse (const bool enabled)
{
    m_pd.m_hasStartReverse = enabled;
}

// Returns true/false, whether or not the interface supports arbitrary acquisition start.
bool ImageView::AbstractAcquisitionInterface::hasStart (void) const
{
    return m_pd.m_hasStart;
}
void ImageView::AbstractAcquisitionInterface::setHasStart (const bool enabled)
{
    m_pd.m_hasStart = enabled;
}

// Returns true/false, whether or not the interface supports acquisition of a fixed count of frames.
bool ImageView::AbstractAcquisitionInterface::hasStartFixed (void) const
{
    return m_pd.m_hasStartFixed;
}
void ImageView::AbstractAcquisitionInterface::setHasStartFixed (const bool enabled)
{
    m_pd.m_hasStartFixed = enabled;
}

// Returns true/false, whether or not the interface supports grabbing a single frame.
bool ImageView::AbstractAcquisitionInterface::hasStartSingle (void) const
{
    return m_pd.m_hasStartSingle;
}
void ImageView::AbstractAcquisitionInterface::setHasStartSingle (const bool enabled)
{
    m_pd.m_hasStartSingle = enabled;
}

// Returns true/false, whether or not the interface supports pausing an active acquisition.
bool ImageView::AbstractAcquisitionInterface::hasPause (void) const
{
    return m_pd.m_hasPause;
}
void ImageView::AbstractAcquisitionInterface::setHasPause (const bool enabled)
{
    m_pd.m_hasPause = enabled;
}

// Returns true/false, whether or not the interface supports arbitrary acquisition stop.
bool ImageView::AbstractAcquisitionInterface::hasStop (void) const
{
    return m_pd.m_hasStop;
}
void ImageView::AbstractAcquisitionInterface::setHasStop (const bool enabled)
{
    m_pd.m_hasStop = enabled;
}

// Returns true/false, whether or not the interface supports frame-rate querying.
bool ImageView::AbstractAcquisitionInterface::hasFrameRate (void) const
{
    return m_pd.m_hasFrameRate;
}
void ImageView::AbstractAcquisitionInterface::setHasFrameRate (const bool enabled)
{
    m_pd.m_hasFrameRate = enabled;
}

// Returns true/false, whether or not the interface supports a queued frame count.
bool ImageView::AbstractAcquisitionInterface::hasFramesQueued (void) const
{
    return m_pd.m_hasFramesQueued;
}
void ImageView::AbstractAcquisitionInterface::setHasFramesQueued (const bool enabled)
{
    m_pd.m_hasFramesQueued = enabled;
}

// Returns true/false, whether or not the interface supports a filled (waiting) frame count.
bool ImageView::AbstractAcquisitionInterface::hasFramesFilled (void) const
{
    return m_pd.m_hasFramesFilled;
}
void ImageView::AbstractAcquisitionInterface::setHasFramesFilled (const bool enabled)
{
    m_pd.m_hasFramesFilled = enabled;
}

// Returns true/false, whether or not the interface supports a dropped frame count.
bool ImageView::AbstractAcquisitionInterface::hasFramesDropped (void) const
{
    return m_pd.m_hasFramesDropped;
}
void ImageView::AbstractAcquisitionInterface::setHasFramesDropped (const bool enabled)
{
    m_pd.m_hasFramesDropped = enabled;
}

// Returns true/false, whether or not the interface supports a delivered frame count.
bool ImageView::AbstractAcquisitionInterface::hasFramesDelivered (void) const
{
    return m_pd.m_hasFramesDelivered;
}
void ImageView::AbstractAcquisitionInterface::setHasFramesDelivered (const bool enabled)
{
    m_pd.m_hasFramesDelivered = enabled;
}

// The minimum startFixed count. Default is 1.
std::int32_t ImageView::AbstractAcquisitionInterface::startFixedMin (void) const
{
    return m_pd.m_startFixedMin;
}
void ImageView::AbstractAcquisitionInterface::setStartFixedMin (const std::int32_t val)
{
    m_pd.m_startFixedMin = val;
}

// The maximum startFixed count. Default is 65536.
std::int32_t ImageView::AbstractAcquisitionInterface::startFixedMax (void) const
{
    return m_pd.m_startFixedMax;
}
void ImageView::AbstractAcquisitionInterface::setStartFixedMax (const std::int32_t val)
{
    m_pd.m_startFixedMax = val;
}

// The startFixed count increment, from startFixedMin. Default is 1.
std::int32_t ImageView::AbstractAcquisitionInterface::startFixedInc (void) const
{
    return m_pd.m_startFixedInc;
}
void ImageView::AbstractAcquisitionInterface::setStartFixedInc (const std::int32_t val)
{
    m_pd.m_startFixedInc = val;
}

// Setup the interface for acquisition. Base implementation simply returns false.
bool ImageView::AbstractAcquisitionInterface::setup (void)
{
    return false;
}

// Setup the interface for acquisition using a specified mode. Base implementatino simply returns false.
bool ImageView::AbstractAcquisitionInterface::setupMode (const char *const mode)
{
    return false;
}

// Cleanup the acquisition interface. Base implementation simply returns false.
bool ImageView::AbstractAcquisitionInterface::cleanup (void)
{
    return false;
}

// Trigger the acquisition interface. Base implementation simply returns false.
bool ImageView::AbstractAcquisitionInterface::trigger (void)
{
    return false;
}

// Start reverse acquisition of a single frame. Base implementation simply returns false.
bool ImageView::AbstractAcquisitionInterface::startSingleReverse (void)
{
    return false;
}

// Start a reverse acquisition. Base implementation simply returns false.
bool ImageView::AbstractAcquisitionInterface::startReverse (void)
{
    return false;
}

// Start acquisition. Base implementation returns false.
bool ImageView::AbstractAcquisitionInterface::start (void)
{
    return false;
}

// Start a fixed-length acquisition. Base implementation simply returns false.
bool ImageView::AbstractAcquisitionInterface::startFixed (const size_t cnt)
{
    return false;
}

// Start acquisition of a single frame. Base implementation simply returns false.
bool ImageView::AbstractAcquisitionInterface::startSingle (void)
{
    return false;
}

// Pause an acquisition. Base implementation simply returns false.
bool ImageView::AbstractAcquisitionInterface::pause (void)
{
    return false;
}

// Stop acquisition. Base implementation simply returns false.
bool ImageView::AbstractAcquisitionInterface::stop (void)
{
    return false;
}

// Returns true/false, whether or not an acquisition may be setup.
bool ImageView::AbstractAcquisitionInterface::canSetup (void) const
{
    return false;
}

// Returns true/false, whether or not the interface may be setup for acquisition with a mode.
bool ImageView::AbstractAcquisitionInterface::canSetupMode (void) const
{
    return false;
}

// Returns true/false, whether or not the acquisition interface may be cleaned up.
bool ImageView::AbstractAcquisitionInterface::canCleanup (void) const
{
    return false;
}

// Returns true/false, whether or not the interface may be triggered.
bool ImageView::AbstractAcquisitionInterface::canTrigger (void) const
{
    return false;
}

// Returns true/false, whether or not reverse acquisition of a single frame can be started.
bool ImageView::AbstractAcquisitionInterface::canStartSingleReverse (void) const
{
    return false;
}

// Returns true/false, whether or not a continuous reverse acquisition can be started.
bool ImageView::AbstractAcquisitionInterface::canStartReverse (void) const
{
    return false;
}

// Returns true/false, whether or not a continuous acquisition can be started.
bool ImageView::AbstractAcquisitionInterface::canStart (void) const
{
    return false;
}

// Returns true/false, whether or not a fixed-frame acquisition can be started.
bool ImageView::AbstractAcquisitionInterface::canStartFixed (void) const
{
    return false;
}

// Returns true/false, whether or not a single-frame acquisition can be started.
bool ImageView::AbstractAcquisitionInterface::canStartSingle (void) const
{
    return false;
}

// Returns true/false, whether or not acquisition may be paused.
bool ImageView::AbstractAcquisitionInterface::canPause (void) const
{
    return false;
}

// Returns true/false, whether or not acquisition may be stopped.
bool ImageView::AbstractAcquisitionInterface::canStop (void) const
{
    return false;
}

// Returns true/false, whether or not reverse acquisition of a single frame is running. Default returns false.
bool ImageView::AbstractAcquisitionInterface::isRunningSingleReverse (void) const
{
    return false;
}

// Returns true/false, whether or not a reverse acquisition is running. Default returns false.
bool ImageView::AbstractAcquisitionInterface::isRunningReverse (void) const
{
    return false;
}

// Returns true/false, whether or not acquisition is grabbing a fixed-length sequence of frames.
bool ImageView::AbstractAcquisitionInterface::isRunningFixed (void) const
{
    return false;
}

// Returns true/false, whether or not acquisition is grabbing a single frame.
bool ImageView::AbstractAcquisitionInterface::isRunningSingle(void) const
{
    return false;
}

// Returns true/false, whether or not acquisition is running. Default returns false.
bool ImageView::AbstractAcquisitionInterface::isPaused (void) const
{
    return false;
}

// Returns true/false, whether or not acquisition is running. Default returns !isRunning().
bool ImageView::AbstractAcquisitionInterface::isStopped (void) const
{
    return !isRunning();
}

//// Setup mode.

// Add a mode for acquisition setup.
bool ImageView::AbstractAcquisitionInterface::addSetupMode (const char *const mode)
{
    return addSetupMode(mode, "");
}

bool ImageView::AbstractAcquisitionInterface::addSetupMode (const char *const mode, const char *const description)
{
    if (!mode || !description)
        return false;

    const auto extantIter = m_pd.m_setupModesMap.find(mode);
    if (m_pd.m_setupModesMap.end() != extantIter)
        return false;

    const PrivateData::StrPair modeDescPair (mode, description);
    m_pd.m_setupModesMap.insert(modeDescPair);
    m_pd.m_setupModesVector.push_back(modeDescPair);

    return true;
}

// Clear all previously added setup modes.
void ImageView::AbstractAcquisitionInterface::clearSetupModes (void)
{
    m_pd.m_setupModesMap.clear();
    m_pd.m_setupModesVector.clear();
}

// The number of added setup modes.
size_t ImageView::AbstractAcquisitionInterface::setupModeCount (void) const
{
    return m_pd.m_setupModesVector.size();
}

// Return the setup mode for the given index.
bool ImageView::AbstractAcquisitionInterface::setupModeName (const size_t index, char *const pModeStr, size_t *const pSize) const
{
    if (m_pd.m_setupModesVector.size() <= index)
        return false;

    return ImageView::String::set(m_pd.m_setupModesVector[index].first, pModeStr, pSize);
}

// Return the setup mode description for the given index.
bool ImageView::AbstractAcquisitionInterface::setupModeDescription (const size_t index, char *const pDescStr, size_t *const pSize) const
{
    if (m_pd.m_setupModesVector.size() <= index)
        return false;

    return ImageView::String::set(m_pd.m_setupModesVector[index].second, pDescStr, pSize);
}

// Return the name of the setup mode, if any. Default simply returns false.
bool ImageView::AbstractAcquisitionInterface::activeSetupMode (char *const pModeStr, size_t *const pSize) const
{
    return false;
}

// Return the acquisition frame rate. Base implementation simply returns 0.
double ImageView::AbstractAcquisitionInterface::frameRate (void) const
{
    return 0.;
}

// Return the frame buffer status. Base implementations simply return 0.
size_t ImageView::AbstractAcquisitionInterface::framesQueued (void) const
{
    return 0;
}
size_t ImageView::AbstractAcquisitionInterface::framesFilled (void) const
{
    return 0;
}
size_t ImageView::AbstractAcquisitionInterface::framesDropped (void) const
{
    return 0;
}
size_t ImageView::AbstractAcquisitionInterface::framesDelivered (void) const
{
    return 0;
}

//// ScopedAcquisitionInterface
ImageView::ScopedAcquisitionInterface::ScopedAcquisitionInterface (void)
    : m_acqIf (nullptr)
{ }
ImageView::ScopedAcquisitionInterface::ScopedAcquisitionInterface (AbstractAcquisitionInterface *const acqIf)
    : m_acqIf (acqIf)
{ }

ImageView::ScopedAcquisitionInterface::ScopedAcquisitionInterface (ScopedAcquisitionInterface &&to_take)
    : m_acqIf (to_take.m_acqIf)
{
    to_take.m_acqIf = nullptr;
}

ImageView::ScopedAcquisitionInterface::~ScopedAcquisitionInterface (void)
{
    delete m_acqIf;
}

ImageView::ScopedAcquisitionInterface::operator bool (void) const
{
    return !!m_acqIf;
}
bool ImageView::ScopedAcquisitionInterface::operator! (void) const
{
    return !m_acqIf;
}

void ImageView::ScopedAcquisitionInterface::take (ScopedAcquisitionInterface &&to_take)
{
    delete m_acqIf;
    m_acqIf = to_take.m_acqIf;
    to_take.m_acqIf = nullptr;
}
void ImageView::ScopedAcquisitionInterface::reset (AbstractAcquisitionInterface *const acqIf)
{
    delete m_acqIf;
    m_acqIf = acqIf;
}

ImageView::AbstractAcquisitionInterface* ImageView::ScopedAcquisitionInterface::get (void) const
{
    return m_acqIf;
}
ImageView::AbstractAcquisitionInterface* ImageView::ScopedAcquisitionInterface::operator-> (void) const
{
    return m_acqIf;
}
ImageView::AbstractAcquisitionInterface& ImageView::ScopedAcquisitionInterface::operator* (void) const
{
    return *m_acqIf;
}
