#include "AbstractActionInterface_Private.h"

/* FILE:        AbstractActionInterface.cpp
 * DATE:        8/12/2016
 * AUTHOR:      Jeremy Greene
 * COMPANY:     BitFlow, Inc.
 * COPYRIGHT:   Copyright (C) 2016, BitFlow, Inc.
 * DESCRIPTION: Abstract interface for ImageView GUI actions.
 */


 // Private methods.
void ImageView::AbstractActionInterface::setAction (ImageView::Action *const a_action)
{
    m_pd.m_action = a_action;
}

ImageView::AbstractActionInterface::AbstractActionInterface (void)
    : m_pd (*new PrivateData)
{
}

ImageView::AbstractActionInterface::~AbstractActionInterface (void)
{
    delete &m_pd;
}

// Retrieve the associated ImageView::Action, or nullptr.
ImageView::Action* ImageView::AbstractActionInterface::action (void)
{
    return m_pd.m_action;
}
const ImageView::Action* ImageView::AbstractActionInterface::action (void) const
{
    return m_pd.m_action;
}

// Callback invoked when the action GUI is about to be destroyed. Default does nothing.
void ImageView::AbstractActionInterface::uxDestroyed (void)
{
    // NoOp.
}

// Callback invoked when the action has been triggered by the user. Default does nothing.
void ImageView::AbstractActionInterface::triggered (const bool checked)
{
    // NoOp.
}

ImageView::ScopedActionInterface::ScopedActionInterface (void)
    : m_actionIf (nullptr)
{
}
ImageView::ScopedActionInterface::ScopedActionInterface (AbstractActionInterface *const a_actionIf)
    : m_actionIf (a_actionIf)
{
}
ImageView::ScopedActionInterface::ScopedActionInterface (ScopedActionInterface &&to_take)
    : m_actionIf (to_take.m_actionIf)
{
    to_take.m_actionIf = nullptr;
}

ImageView::ScopedActionInterface::~ScopedActionInterface (void)
{
    delete m_actionIf;
}

ImageView::ScopedActionInterface::operator bool (void) const
{
    return !!m_actionIf;
}
bool ImageView::ScopedActionInterface::operator! (void) const
{
    return !m_actionIf;
}

void ImageView::ScopedActionInterface::take (ScopedActionInterface &&to_take)
{
    if (m_actionIf != to_take.m_actionIf)
    {
        delete m_actionIf;
        m_actionIf = to_take.m_actionIf;
        to_take.m_actionIf = nullptr;
    }
}
void ImageView::ScopedActionInterface::reset (AbstractActionInterface *const a_actionIf)
{
    delete m_actionIf;
    m_actionIf = a_actionIf;
}

ImageView::AbstractActionInterface* ImageView::ScopedActionInterface::get (void) const
{
    return m_actionIf;
}
ImageView::AbstractActionInterface* ImageView::ScopedActionInterface::operator-> (void) const
{
    return m_actionIf;
}
ImageView::AbstractActionInterface& ImageView::ScopedActionInterface::operator* (void) const
{
    return *m_actionIf;
}
