#ifndef INCLUDED__BIT_FLOW__IMAGE__VIEW__ABSTRACT__ACTION__INTERFACE__H
#define INCLUDED__BIT_FLOW__IMAGE__VIEW__ABSTRACT__ACTION__INTERFACE__H

/* FILE:        AbstractActionInterface.h
 * DATE:        8/12/2016
 * AUTHOR:      Jeremy Greene
 * COMPANY:     BitFlow, Inc.
 * COPYRIGHT:   Copyright (C) 2016, BitFlow, Inc.
 * DESCRIPTION: Abstract interface for ImageView GUI actions.
 */

#include "ImageView.h"
#include "ImageViewBaseClass.h"

namespace ImageView
{
    class IMAGEVIEW_EXPORT AbstractActionInterface : public BaseClass
    {
    private:
        struct PrivateData;
        PrivateData &m_pd;

        // Private methods.
        void setAction (ImageView::Action *const a_action);

        // Illegal.
        AbstractActionInterface (AbstractActionInterface const&);
        AbstractActionInterface& operator= (AbstractActionInterface const&);

        // Friends.
        friend class ImageView::Action;

    protected:
        AbstractActionInterface (void);

    public:
        virtual ~AbstractActionInterface (void);

        // Retrieve the associated ImageView::Action, or nullptr.
        ImageView::Action* action (void);
        const ImageView::Action* action (void) const;

        // Callback invoked when the action GUI is about to be destroyed. Default does nothing.
        virtual void uxDestroyed (void);

        // Callback invoked when the action has been triggered by the user. Default does nothing.
        virtual void triggered (const bool checked);
    };

    class IMAGEVIEW_EXPORT ScopedActionInterface
    {
    private:
        AbstractActionInterface *m_actionIf;

        // Illegal.
        ScopedActionInterface (int);
        ScopedActionInterface (std::nullptr_t);
        ScopedActionInterface (ScopedActionInterface const&);
        ScopedActionInterface& operator= (ScopedActionInterface const&);

    public:
        explicit ScopedActionInterface (void);
        explicit ScopedActionInterface (AbstractActionInterface *const a_actionIf);
        ScopedActionInterface (ScopedActionInterface &&to_take);

        ~ScopedActionInterface (void);

        operator bool (void) const;
        bool operator! (void) const;

        void take (ScopedActionInterface &&to_take);
        void reset (AbstractActionInterface *const a_actionIf = nullptr);

        AbstractActionInterface* get (void) const;
        AbstractActionInterface* operator-> (void) const;
        AbstractActionInterface& operator* (void) const;
    };

    inline static ScopedActionInterface make_scoped (AbstractActionInterface *const actionIf)
    {
        return ScopedActionInterface(actionIf);
    }
}

#endif // INCLUDED__BIT_FLOW__IMAGE__VIEW__ABSTRACT__ACTION__INTERFACE__H
