#ifndef INCLUDED__IMAGE__VIEW__ABSTRACT__FILE__INTERFACE_____PRIVATE__H
#define INCLUDED__IMAGE__VIEW__ABSTRACT__FILE__INTERFACE_____PRIVATE__H

/* FILE:        AbstractFileInterface_Private.h
 * DATE:        1/7/2016
 * AUTHOR:      Jeremy Greene
 * COMPANY:     BitFlow, Inc.
 * COPYRIGHT:   Copyright (C) 2016, BitFlow, Inc.
 * DESCRIPTION: PrivateData for ImageView::AbstractFileInterface.
 */

#include "AbstractFileInterface.h"

#include <string>
#include <map>

#include <mutex>

struct ImageView::AbstractFileInterface::PrivateData
{
    static const std::string valid_extension_chars;

    struct FileFormat
    {
        std::string primaryExtension;
        std::map<std::string, std::string> alternateExtensions;
        std::string description;
        std::string pattern;
    };
    typedef std::map<std::string, FileFormat> FileFormatMap;

    AbstractFileInterface &m_fileIf;

    FileFormatMap m_fileFormats;
    std::mutex m_fileFormatsMutex;

    // Constructor.
    PrivateData (AbstractFileInterface &fileIf);

    //// Helpers.
    static std::string cleanupFileExtension (const char *const extension);
    static std::string comparableFileExtension (const char *const extension);
    static bool parseFileFormat (const char *const primaryExtension, const char *const alternateExtensions, const char *const description, FileFormat *const format);
};

#endif // INCLUDED__IMAGE__VIEW__ABSTRACT__FILE__INTERFACE_____PRIVATE__H
