/* FILE:        AcquisitionSignalHandler.cpp
 * DATE:        1/7/2016
 * AUTHOR:      Jeremy Greene
 * COMPANY:     BitFlow, Inc.
 * COPYRIGHT:   Copyright (C) 2016, BitFlow, Inc.
 * DESCRIPTION: Implementation of ImageView::AcquisitionSignalHandler.
 */

#include "AcquisitionSignalHandler.h"

ImageView::AcquisitionSignalHandler::AcquisitionSignalHandler (ImageView::AbstractAcquisitionInterface &acqIf)
    : m_acqIf (acqIf)
{ }

void ImageView::AcquisitionSignalHandler::emitFeatureSetChanged (void)
{
    emit featureSetChanged();
}

void ImageView::AcquisitionSignalHandler::emitAcquisitionStateChanged (void)
{
    emit acquisitionStateChanged();
}

void ImageView::AcquisitionSignalHandler::emitGetFrameNow (void)
{
    emit getFrameNow();
}

// Setup the interface for acquisition. Base implementation simply returns false.
bool ImageView::AcquisitionSignalHandler::setup (void)
{
    return m_acqIf.setup();
}

// Cleanup the acquisition interface. Base implementation simply returns false.
bool ImageView::AcquisitionSignalHandler::cleanup (void)
{
    return m_acqIf.cleanup();
}

// Start reverse acquisition of a single frame. Base implementation simply returns false.
bool ImageView::AcquisitionSignalHandler::startSingleReverse (void)
{
    return m_acqIf.startSingleReverse();
}

// Start reverse acquisition. Base implementation simply returns false.
bool ImageView::AcquisitionSignalHandler::startReverse (void)
{
    return m_acqIf.startReverse();
}

// Start acquisition. Base implementation simply returns false.
bool ImageView::AcquisitionSignalHandler::start (void)
{
    return m_acqIf.start();
}

// Start a fixed-length acquisition. Base implementation simply returns false.
bool ImageView::AcquisitionSignalHandler::startFixed (const size_t cnt)
{
    return m_acqIf.startFixed(cnt);
}

// Start acquisition of a single frame. Base implementation simply returns false.
bool ImageView::AcquisitionSignalHandler::startSingle (void)
{
    return m_acqIf.startSingle();
}

// Pause an acquisition. Base implementation simply returns false.
bool ImageView::AcquisitionSignalHandler::pause (void)
{
    return m_acqIf.pause();
}

// Stop acquisition. Base implementation simply returns false.
bool ImageView::AcquisitionSignalHandler::stop (void)
{
    return m_acqIf.stop();
}
