#ifndef INCLUDED__IMAGE__VIEW__ACQUISITION__SIGNAL__HANDLER__H
#define INCLUDED__IMAGE__VIEW__ACQUISITION__SIGNAL__HANDLER__H

/* FILE:        AcquisitionSignalHandler.h
 * DATE:        1/7/2016
 * AUTHOR:      Jeremy Greene
 * COMPANY:     BitFlow, Inc.
 * COPYRIGHT:   Copyright (C) 2016, BitFlow, Inc.
 * DESCRIPTION: Signaling class for ImageView::AbstractAcquisitionInterface.
 */

#include "AbstractAcquisitionInterface.h"

#include <QObject>

class ImageView::AcquisitionSignalHandler : public QObject
{
    Q_OBJECT

private:
    ImageView::AbstractAcquisitionInterface &m_acqIf;

signals:
    // Signal emitted to notify the viewer of a change in the supported features.
    void featureSetChanged (void);

    // Signal emitted to notify the viewer of a change in the acquisition state.
    void acquisitionStateChanged (void);

    // Signal emitted to notify the viewer that it should get a new frame immediately.
    void getFrameNow (void);

public:
    AcquisitionSignalHandler (ImageView::AbstractAcquisitionInterface &acqIf);

    void emitFeatureSetChanged (void);
    void emitAcquisitionStateChanged (void);
    void emitGetFrameNow (void);

public slots:
    // Setup the interface for acquisition. Base implementation simply returns false.
    bool setup (void);

    // Cleanup the acquisition interface. Base implementation simply returns false.
    bool cleanup (void);

    // Start reverse acquisition of a single frame. Base implementation simply returns false.
    bool startSingleReverse (void);

    // Start reverse acquisition. Base implementation simply returns false.
    bool startReverse (void);

    // Start acquisition. Base implementation simply returns false.
    bool start (void);

    // Start a fixed-length acquisition. Base implementation simply returns false.
    bool startFixed (const size_t cnt);

    // Start acquisition of a single frame. Base implementation simply returns false.
    bool startSingle (void);

    // Pause an acquisition. Base implementation simply returns false.
    bool pause (void);

    // Stop acquisition. Base implementation simply returns false.
    bool stop (void);
};

#endif // INCLUDED__IMAGE__VIEW__ACQUISITION__SIGNAL__HANDLER__H
