/* FILE:        ImageViewAbstractLock.cpp
 * DATE:        1/7/2016
 * AUTHOR:      Jeremy Greene
 * COMPANY:     BitFlow, Inc.
 * COPYRIGHT:   Copyright (C) 2016, BitFlow, Inc.
 * DESCRIPTION: Implementation of ImageView::AbstractLock.
 */

#include "ImageViewAbstractLock.h"

#include <utility>

ImageView::AbstractLock::AbstractLock (void)
{ }
ImageView::AbstractLock::AbstractLock (AbstractLock const&)
    : BaseClass()
{ }
ImageView::AbstractLock& ImageView::AbstractLock::operator= (AbstractLock const&)
{
    return *this;
}

ImageView::AbstractLock::~AbstractLock (void) { }

ImageView::ScopedLock::ScopedLock (void)
    : m_cs (nullptr)
{ }
ImageView::ScopedLock::ScopedLock (AbstractLock *const cs)
    : m_cs (cs)
{ }

ImageView::ScopedLock::ScopedLock (ScopedLock &&to_take)
    : m_cs (to_take.m_cs)
{
    to_take.m_cs = nullptr;
}

ImageView::ScopedLock::~ScopedLock (void)
{
    delete m_cs;
}

ImageView::ScopedLock& ImageView::ScopedLock::operator= (ScopedLock &&to_take)
{
    take(std::move(to_take));
    return *this;
}

void ImageView::ScopedLock::take (ScopedLock &&to_take)
{
    if (this != &to_take)
    {
        delete m_cs;
        m_cs = to_take.m_cs;
        to_take.m_cs = nullptr;
    }
}
void ImageView::ScopedLock::reset (AbstractLock *const cs)
{
    delete m_cs;
    m_cs = cs;
}

bool ImageView::ScopedLock::good (void) const
{
    return !!m_cs;
}
ImageView::ScopedLock::operator bool (void) const
{
    return !!m_cs;
}

bool ImageView::ScopedLock::bad (void) const
{
    return !m_cs;
}
bool ImageView::ScopedLock::operator! (void) const
{
    return !m_cs;
}
