#ifndef INCLUDED_BIT_FLOW__IMAGE__VIEW__CRITICAL_SECTION__H
#define INCLUDED_BIT_FLOW__IMAGE__VIEW__CRITICAL_SECTION__H

/* FILE:        ImageViewAbstractLock.h
 * DATE:        1/7/2016
 * AUTHOR:      Jeremy Greene
 * COMPANY:     BitFlow, Inc.
 * COPYRIGHT:   Copyright (C) 2016, BitFlow, Inc.
 * DESCRIPTION: Abstract lock interface for ImageView library multi-threading.
 */

#include "ImageView.h"
#include "ImageViewBaseClass.h"

namespace ImageView
{
    class IMAGEVIEW_EXPORT AbstractLock : public BaseClass
    {
    protected:
        // NoOp.
        AbstractLock (void);
        AbstractLock (AbstractLock const&);
        AbstractLock& operator= (AbstractLock const&);

    public:
        virtual ~AbstractLock (void);
    };

    class IMAGEVIEW_EXPORT ScopedLock
    {
    private:
        ScopedLock (int);
        ScopedLock (std::nullptr_t);
        ScopedLock (ScopedLock const&);
        ScopedLock& operator= (ScopedLock const&);

        AbstractLock *m_cs;

    public:
        explicit ScopedLock (void);
        explicit ScopedLock (AbstractLock *const cs);
        ScopedLock (ScopedLock &&to_take);

        ~ScopedLock (void);

        ScopedLock& operator= (ScopedLock &&to_take);

        void take (ScopedLock &&to_take);
        void reset (AbstractLock *const cs = nullptr);

        bool good (void) const;
        operator bool (void) const;

        bool bad (void) const;
        bool operator! (void) const;
    };

    inline static ScopedLock make_scoped (AbstractLock *const cs)
    {
        return ScopedLock(cs);
    }
}

#endif // INCLUDED_BIT_FLOW__IMAGE__VIEW__CRITICAL_SECTION__H
