#ifndef INCLUDED__BIT_FLOW__IMAGE__VIEW__ACTION__H
#define INCLUDED__BIT_FLOW__IMAGE__VIEW__ACTION__H

/* FILE:        ImageViewAction.h
 * DATE:        8/11/2016
 * AUTHOR:      Jeremy Greene
 * COMPANY:     BitFlow, Inc.
 * COPYRIGHT:   Copyright (C) 2016, BitFlow, Inc.
 * DESCRIPTION: Public display Action class for the ImageView library. Used to interface
 *              with custom ImageView::Window menu and tool-bar entries.
 */

#include "ImageView.h"
#include "ImageViewBaseClass.h"

#include "ImageViewNative.h"

namespace ImageView
{
    class IMAGEVIEW_EXPORT Action : public BaseClass
    {
    private:
        // Private data.
        struct PrivateData;
        PrivateData &m_pd;

        // Illegal.
        Action (void);
        Action (Action const&);
        Action& operator= (Action const&);

    public:
        // Construct an action. Generally, ImageView::Actions are not created explicitly, but by
        // invoking one of the Window::add*Action methods.
        Action (ImageView::Window *const a_window, QAction *const a_action, ImageView::ScopedActionInterface &&a_scopedActionInterface);

        // Destructor. The action may be deleted explicitly from user code, but will otherwise be
        // destroyed automatically when the interface action is destroyed. Only delete an action
        // from the ImageView main thread.
        ~Action (void);

        // Retrieve the containing window handle.
        ImageView::Window* window (void);
        const ImageView::Window* window (void) const;

        // Retrieve the Qt action handle for the underlying QAction.
        QAction *qt_handle (void);
        const QAction *qt_handle (void) const;

        // Retrieve the AbstractActionInterface.
        ImageView::AbstractActionInterface* actionInterface (void);
        const ImageView::AbstractActionInterface* actionInterface (void) const;

        // Action properties. These should only be accessed/modified from the ImageView main thread.
        bool text (char *const pOutStr, size_t *const pOutStrSize) const;
        void setText (const char *const str);

        bool toolTip (char *const pOutStr, size_t *const pOutStrSize) const;
        void setToolTip (const char *const str);

        bool isCheckable (void) const;
        void setCheckable (const bool enabled);

        bool isChecked (void) const;
        void setChecked (const bool enabled);

        bool isEnabled (void) const;
        void setEnabled (const bool enabled);
    };
}

#endif // INCLUDED__BIT_FLOW__IMAGE__VIEW__ACTION__H
