#ifndef INCLUDED__BIT_FLOW__IMAGE__VIEW__APPLICATION__H
#define INCLUDED__BIT_FLOW__IMAGE__VIEW__APPLICATION__H

/* FILE:        ImageViewApplication.h
 * DATE:        11/4/2015
 * AUTHOR:      Jeremy Greene
 * COMPANY:     BitFlow, Inc.
 * COPYRIGHT:   Copyright (C) 2015, BitFlow, Inc.
 * DESCRIPTION: Definition of the ImageView Application class, exactly one instance
 *              of which should be allocated before using any other ImageView classes.
 */

#include "ImageView.h"
#include "ImageViewBaseClass.h"

#include <cstdlib>

namespace ImageView
{
    class IMAGEVIEW_EXPORT Application : public BaseClass
    {
    private:
        struct PrivateData;
        PrivateData *m_pd;

        // Illegal.
        Application (void);
        Application (Application const&);
        Application& operator= (Application const&);

    public:
        // Retrieve the global Application handle. The handle must already have been constructed.
        static Application* global (void);

        // Construct the application. Throw a runtime_error, if the application already exists.
        Application (const int argc, const char *const argv[]);

        // Deconstructor
        ~Application (void);

        // Add a method to be called whenever the event loop is idle. This should only
        // be executed from within the exec thread.
        void addIdleMethod (void(*idleMethod)(void*), void *const userData = nullptr);

        // Whether or not we should exit exec when the last window is closed.
        bool quitOnLastWindowClosed (void) const;
        void setQuitOnLastWindowClosed (const bool enabled);

        // Execute the application main loop. This will not return until the application
        // is ready to exit. This should generally only be executed in the main thread.
        int exec (void);

        // Exit the application main loop (the exec method), returning the given return code.
        void exit (const int returnCode = EXIT_SUCCESS);
    };
}

#endif // INCLUDED__BIT_FLOW__IMAGE__VIEW__APPLICATION__H
