#ifndef INCLUDED__BIT_FLOW__IMAGE__VIEW__APPLICATION_____PRIVATE__H
#define INCLUDED__BIT_FLOW__IMAGE__VIEW__APPLICATION_____PRIVATE__H

/* FILE:        ImageViewApplication_Private.h
 * DATE:        1/7/2016
 * AUTHOR:      Jeremy Greene
 * COMPANY:     BitFlow, Inc.
 * COPYRIGHT:   Copyright (C) 2016, BitFlow, Inc.
 * DESCRIPTION: PrivateData for ImageView::Application.
 */

#include "ImageViewApplication.h"

#include <atomic>
#include <memory>
#include <mutex>

#include <vector>
#include <cstring>

#include <algorithm>

#include <QApplication>
#include <QTimer>

struct ImageView::Application::PrivateData
{
    typedef std::vector<char> ArgvVal;
    typedef std::vector<ArgvVal> ArgvVals;
    typedef std::vector<char*> Argv;

    typedef void(*IdleFunction)(void*);
    typedef std::pair<IdleFunction, void*> IdlePair;
    typedef std::vector<IdlePair> IdleFunctionPairs;

    static std::atomic<Application*> globalApplication;

    int m_argc;
    ArgvVals m_argvVals;
    Argv m_argv;

    std::unique_ptr<QApplication> m_qApplication;

    IdleFunctionPairs m_idleFunctionPairs;
    QTimer *m_idleTimer;

    PrivateData (const int argc, const char *const argv[]);
    ~PrivateData (void);

};

#endif // INCLUDED__BIT_FLOW__IMAGE__VIEW__APPLICATION_____PRIVATE__H
