#ifndef INCLUDED__BIT_FLOW__IMAGE__VIEW__C__DEF__H
#define INCLUDED__BIT_FLOW__IMAGE__VIEW__C__DEF__H

/* FILE:        ImageViewCApi.h
 * DATE:        11/23/2015
 * AUTHOR:      Jeremy Greene
 * COMPANY:     BitFlow, Inc.
 * COPYRIGHT:   Copyright (C) 2015, BitFlow, Inc.
 * DESCRIPTION: Pure C API interface to the ImageView library. Structures and static constants are defined here.
 */

#ifdef __cplusplus
extern "C" {
#endif  /* __cplusplus */

#include <stddef.h>
#include <stdint.h>

#include "ImageViewNative.h"

#if defined(_WIN32)
#   ifdef IMAGEVIEW_LIB
#       define IV_IMPORT_EXPORT __declspec(dllexport)
#   else
#       define IV_IMPORT_EXPORT __declspec(dllimport)
#   endif
#   define IV_API __cdecl
#elif defined(__GNUC__)
#   ifdef IMAGEVIEW_LIB
#       define IV_IMPORT_EXPORT
#   else
#       define IV_IMPORT_EXPORT
#   endif
#   if defined(__x86_64)
#       define IV_API
#   else
#       define IV_API __attribute__ ((cdecl))
#   endif
#else
#   error No calling convention available for this platform.
#endif

typedef enum IVRCEnum
{
    IV_OK = 0,

    // Universal errors.
    IV_ERR_UNKNOWN_ERROR = -1000,
    IV_ERR_NULL_ARGUMENT = -1001,
    IV_ERR_NULL_HANDLE = -1002,
    IV_ERR_NULL_HANDLE_REF = -1003,
    IV_ERR_BAD_HANDLE = -1004,
    IV_ERR_APPLICATION_NOT_CREATED = -1005,
    IV_ERR_BUFFER_TOO_SMALL = -1006,

    // IVAppCreate errors.
    IV_ERR_APPLICATION_ALREADY_CREATED = -2000,
    IV_ERR_APPLICATION_CREATION_FAILED = -2001,

    // IVApp IVSetProp*/IVGetProp*
    IV_ERR_APPLICATION_PROPERTY_CODE_UNKNOWN = -2100,

    // IVWindow.
    IV_ERR_WINDOW_NOT_GOOD = -3000,

    // IVWindow IVSendSignal.
    IV_ERR_WINDOW_SIGNAL_CODE_UNKNOWN = -3100,

    // IVWindow IVSetProp*/IVGetProp*
    IV_ERR_WINDOW_PROPERTY_CODE_UNKNOWN = -3200,
    IV_ERR_WINDOW_BAYER_FILTER_UNKNOWN = -3201,
    IV_ERR_WINDOW_COLOR_TABLE_UNKNOWN = -3202,
    IV_ERR_WINDOW_BAD_GEOMETRY = -3203,

    // IVWindow IVSetFeature*
    IV_ERR_WINDOW_FEATURE_CODE_UNKNOWN = -3300,

    // IVWindow setup mode errors.
    IV_ERR_WINDOW_BAD_SETUP_MODE = -3400,

    // IVWindow file format errors.
    IV_ERR_WINDOW_BAD_FILE_FORMAT = -3500,

    // IVDialogButton errors.
    IV_ERR_DIALOG_BUTTON_UNKNOWN_BUTTONS = -3600,
    IV_ERR_DIALOG_BUTTON_BAD_DEFAULT = -3601,

    // IVMessageBoxDialog errors.
    IV_ERR_MESSAGE_BOX_DIALOG_TYPE_UNKNOWN = -3700,

    // IVWindow add action errors.
    IV_ERR_WINDOW_ACTION_NULL_MENU_NAME = -3800,
    IV_ERR_WINDOW_ACTION_UNEXPECTED_MENU_NAME = -3801,
    IV_ERR_WINDOW_ACTION_UNKNOWN_ANCHOR = -3802,

    // IVAction IVSetProp*/IVGetProp*
    IV_ERR_ACTION_PROPERTY_CODE_UNKNOWN = -4000,

    // IVAction IVSetFeature*
    IV_ERR_ACTION_FEATURE_CODE_UNKNOWN = -4100
} IVRC;

typedef enum IVFormatEnum
{
    IV_FMT_UNKNOWN = 0,

    // Base formats.
    IV_FMT_GRAY8 = 8,
    IV_FMT_GRAY10 = 10,
    IV_FMT_GRAY12 = 12,
    IV_FMT_GRAY14 = 14,
    IV_FMT_GRAY16 = 16,

    IV_FMT_COLOR24 = 24,
    IV_FMT_COLOR30 = 30,
    IV_FMT_COLOR32 = 32,

    IV_FMT_HI_COLOR36 = 36,
    IV_FMT_HI_COLOR42 = 42,
    IV_FMT_HI_COLOR48 = 48,

    // Encoding flags.
    IV_FMT_UNENCODED = 0,
    IV_FMT_BAYER_GBRG = 0x00000100,
    IV_FMT_BAYER_GRBG = 0x00000200,
    IV_FMT_BAYER_RGGB = 0x00000300,
    IV_FMT_BAYER_BGGR = 0x00000400,

    IV_FMT_COLOR_ORDER_RGB = 0x00001000, // Default
    IV_FMT_COLOR_ORDER_BGR = 0x00002000,

    IV_FMT_PACKED = 0x00004000,
    IV_FMT_PACKED_NO_SWAP = 0x00008000,

    // Masks.
    IV_FMT_MASK_BASE = 0x000000FF,
    IV_FMT_MASK_BAYER = 0x00000F00,
    IV_FMT_MASK_COLOR_ORDER = 0x00003000,
    IV_FMT_MASK_PACKED = 0x0000C000,
    IV_FMT_MASK_STRUCTURAL = IV_FMT_MASK_BASE | IV_FMT_MASK_PACKED,
    IV_FMT_MASK_ENCODING = ~IV_FMT_MASK_BASE
} IVFormat;

typedef enum IVWindowColorTableEnum
{
    IV_COLOR_TABLE_GRAY = 0,
    IV_COLOR_TABLE_GRAY2X = 1,
    IV_COLOR_TABLE_GRAY_INVERSE = 2,
    IV_COLOR_TABLE_SINE = 3,
    IV_COLOR_TABLE_COSINE = 4,
    IV_COLOR_TABLE_RAINBOW = 5,
    IV_COLOR_TABLE_RANDOM = 6,
    IV_COLOR_TABLE_CONTOURS = 7
} IVWindowColorTable;

typedef enum IVWindowActionAnchorEnum
{
    IV_WINDOW_ACTION_MAIN_MENU = 0,
    IV_WINDOW_ACTION_TOOL_BAR = 1,
    IV_WINDOW_ACTION_IMAGE_STATS = 2
} IVWindowActionAnchor;

typedef enum IVMessageBoxDialogTypeEnum
{
    IV_MESSAGE_TYPE_CRITICAL = 0,
    IV_MESSAGE_TYPE_WARNING = 1,
    IV_MESSAGE_TYPE_INFORMATION = 2,
    IV_MESSAGE_TYPE_QUESTION = 3,
} IVMessageBoxDialogType;

typedef enum IVDialogButtonEnum
{
    IV_DIALOG_BUTTON_NO_BUTTON = 0,
    IV_DIALOG_BUTTON_OK = 1 << 0,
    IV_DIALOG_BUTTON_OPEN = 1 << 1,
    IV_DIALOG_BUTTON_SAVE = 1 << 2,
    IV_DIALOG_BUTTON_CANCEL = 1 << 3,
    IV_DIALOG_BUTTON_CLOSE = 1 << 4,
    IV_DIALOG_BUTTON_DISCARD = 1 << 5,
    IV_DIALOG_BUTTON_APPLY = 1 << 6,
    IV_DIALOG_BUTTON_RESET = 1 << 7,
    IV_DIALOG_BUTTON_RESTORE_DEFAULTS = 1 << 8,
    IV_DIALOG_BUTTON_HELP = 1 << 9,
    IV_DIALOG_BUTTON_SAVE_ALL = 1 << 10,
    IV_DIALOG_BUTTON_YES = 1 << 11,
    IV_DIALOG_BUTTON_YES_TO_ALL = 1 << 12,
    IV_DIALOG_BUTTON_NO = 1 << 13,
    IV_DIALOG_BUTTON_NO_TO_ALL = 1 << 14,
    IV_DIALOG_BUTTON_ABORT = 1 << 15,
    IV_DIALOG_BUTTON_RETRY = 1 << 16,
    IV_DIALOG_BUTTON_IGNORE = 1 << 17
} IVDialogButton;

typedef enum IVAppProperties_bool_Enum
{
    IV_APP_QUIT_ON_LAST_WINDOW_CLOSED = 01001000
} IVAppProperties_bool;

typedef enum IVWindowSignals_Enum
{
    IV_WINDOW_RAISE = 10001000,

    IV_WINDOW_SET_RUNNING = 10001100,
    IV_WINDOW_SET_CONFIGURED = 10001101,
    IV_WINDOW_SET_UNCONFIGURED = 10001102,

    IV_WINDOW_FEATURE_SET_CHANGE = 10001200,
    IV_WINDOW_ACQUISITION_STATE_CHANGE = 10001201,
    IV_WINDOW_GET_FRAME_NOW = 10001202
} IVWindowSignals;

typedef enum IVWindowProperties_bool_Enum
{
    IV_WINDOW_PREVIEW_ZOOM_LEVEL = 10002000,
    IV_WINDOW_PREVIEW_IMAGE_DIMENSIONS = 10002001,
    IV_WINDOW_PREVIEW_IMAGE_FORMAT = 10002002,
    IV_WINDOW_PREVIEW_PIXEL_VALUE = 10002003,
    IV_WINDOW_PREVIEW_FRAME_RATE = 10002004,
    IV_WINDOW_PREVIEW_BUFFERS_FILLED = 10002005,
    IV_WINDOW_PREVIEW_BUFFERS_DROPPED = 10002006,
    IV_WINDOW_PREVIEW_BUFFERS_DELIVERED = 10002007,
    IV_WINDOW_PREVIEW_BUFFERS_QUEUED = 10002008,
    IV_WINDOW_PREVIEW_SETUP_MODE = 10002009,

    IV_WINDOW_KEEP_IMAGE_INFO_VISIBLE = 10002100,
    IV_WINDOW_BAYER_DEMOSAIC_ENABLED = 10002101,
    IV_WINDOW_RGB_ORDER_REVERSED = 1000202,
    IV_WINDOW_PIXEL_CROSSHAIR_VISIBLE = 10002103,
    IV_WINDOW_REFRESH_ENABLED = 10002104,
    IV_WINDOW_REFRESH_RATE_AUTO = 10002105,

    IV_WINDOW_VISIBLE = 10002200,
    IV_WINDOW_CLOSEABLE = 10002201,
    IV_WINDOW_FULL_SCREEN = 10002202,
    IV_WINDOW_MAXIMIZED = 10002203,
    IV_WINDOW_MINIMIZED = 10002204,

    IV_WINDOW_ZOOM_TO_FIT = 10002300,

    // Whether or not displayed image buffers will remain allocated for the life of the Window.
    // If not, than IV_WINDOW_BRIIL_LOCK_IMAGE and IV_WINDOW_BIL_UNLOCK_IMAGE will need to be
    // implemented.
    IV_WINDOW_ACQUISITION_FRAMES_LOCKED = 10002400
} IVWindowProperties_bool;

typedef enum IVWindowProperties_int32_Enum
{
    IV_WINDOW_POS_X = 10003000,
    IV_WINDOW_POS_Y = 10003001,
    IV_WINDOW_SIZE_WIDTH = 10003002,
    IV_WINDOW_SIZE_HEIGHT = 10003003,

    IV_WINDOW_BAYER_FILTER_OVERRIDE = 10003004,
    IV_WINDOW_COLOR_TABLE = 10003005,

    IV_WINDOW_REFRESH_RATE = 10003006,

    IV_WINDOW_IMAGE_OFFSET_X = 10003100,
    IV_WINDOW_IMAGE_OFFSET_Y = 10003101,

#if defined(_WIN32)
    // Setup the application about window icon from the Windows binary resource index, used
    // with MAKEINTRESOURCE. IV_WINDOW_APPLICATION_ABOUT_TITLE or IV_WINDOW_APPLICATION_ABOUT_DESCRIPTION
    // must be set subsequently, to apply the new icon.
    IV_WINDOW_APPLICATION_ABOUT_ICON_RC = 10003200,
#endif

    IV_WINDOW_START_FIXED_MIN = 10004000,
    IV_WINDOW_START_FIXED_MAX = 10004001,
    IV_WINDOW_START_FIXED_INC = 10004002
} IVWindowProperties_int32;

typedef enum IVWindowProperties_double_Enum
{
    IV_WINDOW_ZOOM_LEVEL = 10005000,
} IVWindowProperties_double;

typedef enum IVWindowProperties_cstr_Enum
{
    IV_WINDOW_TITLE = 10006000,

    // Setup the application about window title. If the executable's Windows binary resources
    // provide an icon at MAKEINTRESOURCE(IV_WINDOW_APPLICATION_ABOUT_ICON_RC), that icon
    // is used in the about dialog.
    IV_WINDOW_APPLICATION_ABOUT_TITLE = 10006100,

    // Setup the application about window description (simple HTML supported).
    IV_WINDOW_APPLICATION_ABOUT_DESCRIPTION = 10006101,

    // The window geometry. Use this value to restore window state and location at a later time.
    IV_WINDOW_RESTORE_GEOMETRY = 10007000
} IVWindowProperties_cstr;

typedef enum IVWindowProperties_data_Enum
{
    // Setup the application about window icon from a XPM formatted c-string array.
    // IV_WINDOW_APPLICATION_ABOUT_TITLE or IV_WINDOW_APPLICATION_ABOUT_DESCRIPTION must be set
    // subsequently, to apply the new icon.
    IV_WINDOW_APPLICATION_ABOUT_ICON_XPM = 10008000
} IVWindowProperties_data;

typedef enum IVActionProperties_bool_Enum
{
    IV_ACTION_CHECKABLE = 10010000,
    IV_ACTION_CHECKED = 10010001,
    IV_ACTION_ENABLED = 10010002
} IVActionProperties_bool;

typedef enum IVActionProperties_cstr_Enum
{
    IV_ACTION_TEXT = 10011000,
    IV_ACTION_TOOL_TIP = 10011001
} IVActionProperties_cstr;

typedef enum IVWindowFeatures_void_void_Enum
{
    // The IV_WINDOW_VV_CLOSED feature method is called when the IVWindow has been closed, and is
    // cleaning up its resources. If implemented, the client is responsible for all acquisition
    // cleanup. Otherwise, the IVWindow follows this procedure:
    //    if IV_WINDOW_IS_SETUP then IV_WINDOW_BV_CLEANUP
    IV_WINDOW_VV_CLOSED = 20003000,

    // Invoked whenever the IVWindow is ready to display another image, immediately prior to
    // retrieving the next frame.
    IV_WINDOW_VV_READY_NEXT_FRAME = 20003100
} IVWindowFeatures_void_void;

typedef enum IVWindowFeatures_bool_void_Enum
{
    IV_WINDOW_BV_SETUP = 20004000,
    IV_WINDOW_BV_CLEANUP = 20004001,
    IV_WINDOW_BV_TRIGGER = 20004002,
    IV_WINDOW_BV_START_SINGLE_REVERSE = 20004003,
    IV_WINDOW_BV_START_REVERSE = 20004004,
    IV_WINDOW_BV_START = 20004005,
    IV_WINDOW_BV_START_SINGLE = 20004006,
    IV_WINDOW_BV_PAUSE = 20004007,
    IV_WINDOW_BV_STOP = 20004008,

    IV_WINDOW_BV_CAN_SETUP = 20004100,
    IV_WINDOW_BV_CAN_SETUP_MODE = 20004101,
    IV_WINDOW_BV_CAN_CLEANUP = 20004102,
    IV_WINDOW_BV_CAN_TRIGGER = 20004103,
    IV_WINDOW_BV_CAN_START_SINGLE_REVERSE = 20004104,
    IV_WINDOW_BV_CAN_START_REVERSE = 20004105,
    IV_WINDOW_BV_CAN_START = 20004106,
    IV_WINDOW_BV_CAN_START_SINGLE = 20004107,
    IV_WINDOW_BV_CAN_START_FIXED = 20004108,
    IV_WINDOW_BV_CAN_PAUSE = 20004109,
    IV_WINDOW_BV_CAN_STOP = 20004110,

    IV_WINDOW_BV_IS_SETUP = 20004200, // Required for basic functionality.
    IV_WINDOW_BV_IS_RUNNING_SINGLE_REVERSE = 20004201,
    IV_WINDOW_BV_IS_RUNNING_REVERSE = 20004202,
    IV_WINDOW_BV_IS_RUNNING = 20004203, // Required for basic functionality.
    IV_WINDOW_BV_IS_RUNNING_SINGLE = 20004204,
    IV_WINDOW_BV_IS_RUNNING_FIXED = 20004205,
    IV_WINDOW_BV_IS_PAUSED = 20004206,
    IV_WINDOW_BV_IS_STOPPED = 20004207
} IVWindowFeatures_bool_void;

typedef enum IVWindowFeatures_bool_cCStr_Enum
{
    IV_WINDOW_BCSTR_SETUP_MODE = 20005000
} IVWindowFeatures_bool_cCStr;

typedef enum IVWindowFeatures_bool_size_t_Enum
{
    IV_WINDOW_BST_START_FIXED = 20006000
} IVWindowFeatures_bool_size_t;

typedef enum IVWindowFeatures_double_void_Enum
{
    IV_WINDOW_DV_FRAME_RATE = 20007000
} IVWindowFeatures_double_void;

typedef enum IVWindowFeatures_size_t_void_Enum
{
    IV_WINDOW_STV_FRAMES_QUEUED = 20008000,
    IV_WINDOW_STV_FRAMES_FILLED = 20008001,
    IV_WINDOW_STV_FRAMES_DROPPED = 20008002,
    IV_WINDOW_STV_FRAMES_DELIVERED = 20008003
} IVWindowFeatures_size_t_void;

typedef enum IVWindowFeatures_bool_CStr_psize_t_Enum
{
    IV_WINDOW_BCSTRPST_ACTIVE_SETUP_MODE = 20009000
} IVWindowFeatures_bool_CStr_psize_t;

typedef enum IVWindowFeatures_bool_pIVRawImage_pIVImageLock_Enum
{
    // Retrieve a critical section to protect the allocation of buffers returned by getNextFrame.
    // Return false, and no critical section, if allocation cannot be guaranteed.
    // IV_WINDOW_BIL_UNLOCK_IMAGE must also be implemented, for image locking to work.
    IV_WINDOW_BRIIL_LOCK_IMAGE = 20010000
} IVWindowFeatures_bool_pIVRawImage_pIVImageLock;

typedef enum IVWindowFeatures_bool_pIVImageLock_Enum
{
    // Release an IVImageLock previously retrieve using the IV_WINDOW_BRIIL_LOCK_IMAGE method.
    IV_WINDOW_BIL_UNLOCK_IMAGE = 20011000
} IVWindowFeatures_bool_pIVImageLock;

typedef enum IVWindowFeatures_bool_pIVRawImage_cCStr_Enum
{
    // Save the specified image to the specified file path location. File type derived from extension.
    IV_WINDOW_BRICSTR_IMAGE_FILE_SAVE = 20012000
} IVWindowFeatures_bool_pIVRawImage_cCStr;

typedef enum IVActionFeatures_void_void_Enum
{
    // The IV_ACTION_VV_DESTROYED feature method is called when the IVAction has been destroyed, and
    // is cleaning up its resources.
    IV_ACTION_VV_DESTROYED = 30001000
} IVActionFeatures_void_void;

typedef enum IVActionFeatures_void_bool_Enum
{
    // Invoked when the IVAction has been triggered by the user. If the action is checkable, the first
    // parameter indicates the IVAction checked state resulting from the trigger.
    IV_ACTION_VB_TRIGGERED = 30002000
} IVActionFeatures_void_bool;


//// Type definitions.

typedef void IVvoid;
typedef char IVchar;
typedef uint8_t IVbool;
typedef uint8_t IVuint8;
typedef int32_t IVint32;
typedef uint32_t IVuint32;
typedef double IVdouble;
typedef size_t IVsize_t;

#define IV_NULL (0)
#define IV_FALSE (0)
#define IV_TRUE (!IV_FALSE)

// Handles.
typedef IVvoid *IVHandle;

typedef struct IVAppStruct *IVApp;
typedef struct IVWindowStruct *IVWindow;
typedef struct IVActionStruct *IVAction;

typedef IVvoid *IVUserData;

typedef struct IVRawImageStruct
{
    const IVuint8 *buffer;
    IVsize_t width;
    IVsize_t height;
    IVFormat format;
    IVUserData userData;
} IVRawImage;

typedef struct IVImageLockStruct
{
    IVUserData userData;
} IVImageLock;


//// IVApp method definitions.

typedef IVvoid (IV_API *IVAppIdleMethod) (IVUserData);


//// Feature method types.

typedef IVvoid (IV_API *IVFeature_void_void_Method) (IVUserData);
typedef IVvoid (IV_API *IVFeature_void_bool_Method) (IVUserData, IVbool);
typedef IVbool (IV_API *IVFeature_bool_void_Method) (IVUserData);
typedef IVbool (IV_API *IVFeature_bool_cCStr_Method) (IVUserData, const IVchar*);
typedef IVbool (IV_API *IVFeature_bool_size_t_Method) (IVUserData, IVsize_t);
typedef IVdouble (IV_API *IVFeature_double_void_Method) (IVUserData);
typedef IVsize_t (IV_API *IVFeature_size_t_void_Method) (IVUserData);
typedef IVbool (IV_API *IVFeature_bool_CStr_psize_t_Method) (IVUserData, IVchar*, size_t*);

typedef IVbool (IV_API *IVFeature_bool_pIVRawImage_pIVImageLock_Method) (IVUserData, IVRawImage*, IVImageLock*);
typedef IVbool (IV_API *IVFeature_bool_pIVImageLock_Method) (IVUserData, IVImageLock*);

typedef IVbool (IV_API *IVFeature_bool_pIVRawImage_cCStr_Method) (IVUserData, IVRawImage*, const IVchar*);

#ifdef __cplusplus
}
#endif

#endif // INCLUDED__BIT_FLOW__IMAGE__VIEW__C__DEF__H
