/* FILE:        ImageViewFileIO.cpp
 * DATE:        4/4/2016
 * AUTHOR:      Jeremy Greene
 * COMPANY:     BitFlow, Inc.
 * COPYRIGHT:   Copyright (C) 2016, BitFlow, Inc.
 * DESCRIPTION: Methods for image file input output. Supported file types: JPG, PNG, BMP.
 */

#include "ImageViewFileIO_Private.h"

ImageView::FileIO::FileIO (void)
    : m_pd (new PrivateData)
{
}

ImageView::FileIO::FileIO (RawImage const& a_image)
    : m_pd (new PrivateData)
{
    setData(a_image);
}

ImageView::FileIO::~FileIO (void)
{
    delete m_pd;
}

// Set the image data from a RawImage. The data stored may be a different format than the data set.
bool ImageView::FileIO::setData (RawImage const& a_image)
{
    if (m_pd->m_imageAccessor.QImageFromBuffer(m_pd->m_image, m_pd->m_imageData, a_image))
        return true;

    m_pd->m_image = QImage();
    return false;
}

// Query whether or not this FileIO class has any data.
bool ImageView::FileIO::hasData (void) const
{
    return !m_pd->m_image.isNull();
}

// Retrieve the stored image data. FileIO owns the RawImage data.
bool ImageView::FileIO::getData (RawImage &a_image) const
{
    if (m_pd->m_image.isNull())
        return false;

    return m_pd->m_imageAccessor.NormalizeQImage(m_pd->m_image, m_pd->m_imageData, &a_image);
}

// Save the current image data to the specified path.
bool ImageView::FileIO::save (const char *const a_filePath) const
{
    if (m_pd->m_image.isNull())
        return false;

    if (!PrivateData::isValidExtension(a_filePath))
        return false;

    return m_pd->m_image.save(a_filePath);
}

// Load image data from the specified path.
bool ImageView::FileIO::load (const char *const a_filePath)
{
    if (!PrivateData::isValidExtension(a_filePath))
        return false;

    // Load the image.
    if (!m_pd->m_image.load(a_filePath))
        return false;
    
    return m_pd->m_imageAccessor.NormalizeQImage(m_pd->m_image, m_pd->m_imageData);
}
