#ifndef INCLUDED__BIT_FLOW__IMAGE_VIEW__FILE__I_O__H
#define INCLUDED__BIT_FLOW__IMAGE_VIEW__FILE__I_O__H

/* FILE:        ImageViewFileIO.h
 * DATE:        4/4/2016
 * AUTHOR:      Jeremy Greene
 * COMPANY:     BitFlow, Inc.
 * COPYRIGHT:   Copyright (C) 2016, BitFlow, Inc.
 * DESCRIPTION: Methods for image file input output. Supported file types: JPG, PNG, BMP.
 */

#include "ImageView.h"
#include "ImageViewRawImage.h"

namespace ImageView
{
    class IMAGEVIEW_EXPORT FileIO
    {
    private:
        // PrivateData.
        struct PrivateData;
        PrivateData *m_pd;

        // Illegal.
        FileIO (FileIO const&);
        FileIO& operator= (FileIO const&);

    public:
        FileIO (void);
        FileIO (RawImage const& a_image);

        ~FileIO (void);

        // Set the image data from a RawImage. The data stored may be a different format than the data set.
        bool setData (RawImage const& a_image);

        // Query whether or not this FileIO class has any data.
        bool hasData (void) const;

        // Retrieve the stored image data. FileIO owns the RawImage data.
        bool getData (RawImage &a_image) const;

        // Save the current image data to the specified path.
        bool save (const char *const a_filePath) const;

        // Load image data from the specified path.
        bool load (const char *const a_filePath);
    };
}

#endif // INCLUDED__BIT_FLOW__IMAGE_VIEW__FILE__I_O__H
