#ifndef INCLUDED__BIT_FLOW__IMAGE_VIEW__IMAGE__ACCESSOR__H
#define INCLUDED__BIT_FLOW__IMAGE_VIEW__IMAGE__ACCESSOR__H

/* FILE:        ImageViewImageAccessor.h
 * DATE:        4/4/2016
 * AUTHOR:      Jeremy Greene
 * COMPANY:     BitFlow, Inc.
 * COPYRIGHT:   Copyright (C) 2016, BitFlow, Inc.
 * DESCRIPTION: Methods for image data conversion and access.
 */

#include "ImageView.h"
#include "ImageViewRawImage.h"

#include <QImage>

namespace ImageView
{
    class ImageAccessor
    {
    private:
        // PrivateData.
        struct PrivateData;
        PrivateData *m_pd;

        // Illegal.
        ImageAccessor (ImageAccessor const&);
        ImageAccessor& operator= (ImageAccessor const&);

    public:
        ImageAccessor (void);
        ~ImageAccessor (void);

        // Generate a QImage that is a suitable destination for the RawImage, matching all parameters.
        bool AllocateQImage (QImage &qImage, std::vector<uchar> &qImageData, const int width, const int height, const Format format) const;

        // Convert the QImage t a format that ImageView can use. RawImage (if provided) will be populated appropriately.
        bool NormalizeQImage (QImage &a_qImage, std::vector<uchar> &a_qImageData, RawImage *const a_pRawImage = nullptr) const;

        // Convert the input image buffer to a QImage, using qImageData as the qImage data buffer. This
        // performs a deep copy, and so the external buffer may be deallocated after calling.
        bool QImageFromBuffer (QImage &qImage, std::vector<uchar> &qImageData, RawImage const& rawImage) const;

        // Equivalent to QImageFromBuffer, but qImageData will never be reallocated.
        bool QImageFromBufferConst (QImage &qImage, std::vector<uchar> &qImageData, RawImage const& rawImage) const;

        // Retrieve the buffer pixel value specified.
        bool BufferPixelValue (RawImage const& rawImage, PixPos const& pos, PixVal &val) const;
    };
}

#endif // INCLUDED__BIT_FLOW__IMAGE_VIEW__IMAGE__ACCESSOR__H
