#ifndef INCLUDED__BIT_FLOW__IMAGE__VIEW__MESSAGE__BOX__DIALOG__H
#define INCLUDED__BIT_FLOW__IMAGE__VIEW__MESSAGE__BOX__DIALOG__H

/* FILE:        ImageViewMessageBoxDialog.h
 * DATE:        7/6/2016
 * AUTHOR:      Jeremy Greene
 * COMPANY:     BitFlow, Inc.
 * COPYRIGHT:   Copyright (C) 2016, BitFlow, Inc.
 * DESCRIPTION: Public display MessageBox class for the ImageView library. May only be
 *              used after construction of the ImageView::Application global object.
 *              Use Qt's QMessageBox, for greater versatility.
 */

#include "ImageView.h"
#include "ImageViewBaseClass.h"

namespace ImageView
{
    class IMAGEVIEW_EXPORT MessageBoxDialog : public BaseClass
    {
    private:
        // PrivateData.
        struct PrivateData;

        // Non-constructable class.
        MessageBoxDialog (void);
        MessageBoxDialog (MessageBoxDialog const&);
        MessageBoxDialog& operator= (MessageBoxDialog const&);

    public:
        // Create an error message box, returning the user button selection. This will be
        // a modal dialog of the parent window. The text may contain basic HTML elements.
        static DialogButton critical (Window *const parentWindow, const char *const title, const char *const text, DialogButtons const& buttons = DialogButton::Ok, const DialogButton defaultButton = DialogButton::NoButton);

        // Create a warning message box, returning the user button selection. This will be
        // a modal dialog of the parent window. The text may contain basic HTML elements.
        static DialogButton warning (Window *const parentWindow, const char *const title, const char *const text, DialogButtons const& buttons = DialogButton::Ok, const DialogButton defaultButton = DialogButton::NoButton);

        // Create a notification message box, returning the user button selection. This will be
        // a modal dialog of the parent window. The text may contain basic HTML elements.
        static DialogButton information (Window *const parentWindow, const char *const title, const char *const text, DialogButtons const& buttons = DialogButton::Ok, const DialogButton defaultButton = DialogButton::NoButton);

        // Create an inquiry message box, returning the user button selection. This will be
        // a modal dialog of the parent window. The text may contain basic HTML elements.
        static DialogButton question (Window *const parentWindow, const char *const title, const char *const text, DialogButtons const& buttons = DialogButton::Yes| DialogButton::No, const DialogButton defaultButton = DialogButton::NoButton);
    };
}

#endif // INCLUDED__BIT_FLOW__IMAGE__VIEW__MESSAGE__BOX__DIALOG__H
