/* FILE:        ImageViewMessageBoxDialog_Private.h
 * DATE:        7/6/2016
 * AUTHOR:      Jeremy Greene
 * COMPANY:     BitFlow, Inc.
 * COPYRIGHT:   Copyright (C) 2016, BitFlow, Inc.
 * DESCRIPTION: PrivateData definition for the ImageView MessageBoxDialog.
 */

#ifndef INCLUDED__BIT_FLOW__IMAGE__VIEW__MESSAGE__BOX_____PRIVATE__H
#define INCLUDED__BIT_FLOW__IMAGE__VIEW__MESSAGE__BOX_____PRIVATE__H

#include "ImageViewMessageBoxDialog.h"

#include <QObject>
#include <QMessageBox>

struct MessageBoxDialogSignalBox : public QObject
{
    Q_OBJECT;

signals:
    void critical (void);
    void warning (void);
    void information (void);
    void question (void);
};

struct ImageView::MessageBoxDialog::PrivateData : public QObject
{
    Q_OBJECT;

private:
    // Statics.
    static const std::pair<ImageView::DialogButton, QMessageBox::StandardButton> buttonTypePairs [];

    // Members.
    QWidget *const m_parent;
    const QString m_title;
    const QString m_text;
    const QMessageBox::StandardButtons m_buttons;
    const QMessageBox::StandardButton m_defaultButton;

    QMessageBox::StandardButton m_result;

    MessageBoxDialogSignalBox m_signalBox;

    // Helpers.
    static ImageView::DialogButton qt2iv (const QMessageBox::StandardButton qtButton);
    static QMessageBox::StandardButton iv2qt (const ImageView::DialogButton ivButton);
    static QMessageBox::StandardButtons iv2qt (ImageView::DialogButtons const& ivButtons);

    static bool currentIsGuiThread (void);

public:
    PrivateData (Window *const parentWindow, const char *const title, const char *const text, DialogButtons const& buttons, const DialogButton defaultButton);

    DialogButton critical (void);
    DialogButton warning (void);
    DialogButton information (void);
    DialogButton question (void);

private slots:
    void doCritical (void);
    void doWarning (void);
    void doInformation (void);
    void doQuestion (void);
};

#endif // INCLUDED__BIT_FLOW__IMAGE__VIEW__MESSAGE__BOX_____PRIVATE__H
