#ifndef INCLUDED__BIT_FLOW__IMAGE__VIEW__MINI__MAP__H
#define INCLUDED__BIT_FLOW__IMAGE__VIEW__MINI__MAP__H

/* FILE:        ImageViewMiniMap.h
 * DATE:        12/23/2015
 * AUTHOR:      Jeremy Greene
 * COMPANY:     BitFlow, Inc.
 * COPYRIGHT:   Copyright (C) 2015, BitFlow, Inc.
 * DESCRIPTION: A MiniMap style widget for the ImageView library, allowing an image to
 *              be panned very quickly.
 */

#include <QWidget>

namespace ImageView
{
    class MiniMap : public QWidget
    {
        Q_OBJECT;

        Q_PROPERTY(QSizeF   boundarySize    READ boundarySize   WRITE setBoundarySize);
        Q_PROPERTY(QSizeF   viewSize        READ viewSize       WRITE setViewSize);

        Q_PROPERTY(QPointF  offset          READ offset         WRITE setOffset);
        Q_PROPERTY(double   scaling         READ scaling        WRITE setScaling);

    private:
        struct PrivateData;
        PrivateData &m_pd;

        // Illegal.
        MiniMap (MiniMap const&);
        MiniMap& operator= (MiniMap const&);

    public:
        MiniMap (QWidget *const parent = nullptr);
        ~MiniMap (void);

        // Properties.
        QPixmap pixmap (void) const;
        void setPixmap (QPixmap const& value);

        QSizeF boundarySize (void) const;
        void setBoundarySize (QSizeF const& value);
        void setBoundarySize (const double width, const double height);

        QRectF viewRect (void) const;
        void setViewRect (QRectF const& value);
        void setViewRect (const double x, const double y, const double width, const double height);

        QSizeF viewSize (void) const;
        void setViewSize (QSizeF const& value);
        void setViewSize (const double width, const double height);

        QPointF offset (void) const;
        void setOffset (QPointF const& value);
        void setOffset (const double x, const double y);

        double scaling (void) const;
        void setScaling (const double value);

    public slots:
        void moveOffsetOver (QPointF const& pos);
        void moveOffsetOver (const double x, const double y);

        void moveOffsetUp (void);
        void moveOffsetDown (void);
        void moveOffsetLeft (void);
        void moveOffsetRight (void);

        void pageOffsetUp (void);
        void pageOffsetDown (void);
        void pageOffsetLeft (void);
        void pageOffsetRight (void);

    signals:
        void offsetChanged (const QPointF pos);

        void scrollIn (void);
        void scrollOut (void);

    private:
        // Event overrides.
        void paintEvent (QPaintEvent *const evt) override;

        void showEvent (QShowEvent *const evt) override;
        void resizeEvent (QResizeEvent *const evt) override;

        void enterEvent (QEvent *const evt) override;
        void leaveEvent (QEvent *const evt) override;

        void mousePressEvent (QMouseEvent *const evt) override;
        void mouseMoveEvent (QMouseEvent *const evt) override;
        void mouseReleaseEvent (QMouseEvent *const evt) override;

        void wheelEvent (QWheelEvent *const evt) override;
    };
}

#endif // INCLUDED__BIT_FLOW__IMAGE__VIEW__MINI__MAP__H
