/* FILE:        ImageViewNative_Private.h
 * DATE:        7/15/2016
 * AUTHOR:      Jeremy Greene
 * COMPANY:     BitFlow, Inc.
 * COPYRIGHT:   Copyright (C) 2016, BitFlow, Inc.
 * DESCRIPTION: Tools and definitions for integration with the current operating system.
 */

#ifndef INCLUDED__BIT_FLOW__IMAGE__VIEW__NATIVE_____PRIVATE__H
#define INCLUDED__BIT_FLOW__IMAGE__VIEW__NATIVE_____PRIVATE__H

#include <stddef.h>

#if defined(_WIN32)

#   if !defined(WIN32_LEAN_AND_MEAN)
#       define WIN32_LEAN_AND_MEAN
#   endif
#   if !defined(NOMINMAX)
#       define NOMINMAX
#   endif
#   include <Windows.h>

#   define IMAGE_VIEW_NATIVE_WINDOW HWND
#   define IMAGE_VIEW_NATIVE_ICON HICON

// Win32 specific methods.
namespace ImageView
{
    namespace Native
    {
        namespace Win32
        {
            void SetParentChildWindow (IMAGE_VIEW_NATIVE_WINDOW parent, IMAGE_VIEW_NATIVE_WINDOW child);
            void ClearParentChildWindow (IMAGE_VIEW_NATIVE_WINDOW parent, IMAGE_VIEW_NATIVE_WINDOW child);
        }
    }
}

#elif defined(__GNUC__)

#   define IMAGE_VIEW_NATIVE_WINDOW unsigned long*

#else

#   error ImageView does not support this platform yet.

#endif

// General platform-specific methods.
namespace ImageView
{
    namespace Native
    {
        bool LibraryVersion (int &major, int &minor, bool &debug);
        bool LibraryVersionStr (char *const strBuf, size_t *const strBufSize);

        bool DriverVersion (int &major, int &minor, bool &debug);
        bool DriverVersionStr (char *const strBuf, size_t *const strBufSize);

        void Cleanup (void);
    }
}

#endif // INCLUDED__BIT_FLOW__IMAGE__VIEW__NATIVE_____PRIVATE__H
