#ifndef INCLUDED__BIT_FLOW__IMAGE__VIEW__POPOVER__H
#define INCLUDED__BIT_FLOW__IMAGE__VIEW__POPOVER__H

/* FILE:        ImageViewPopover.h
 * DATE:        12/1/2015
 * AUTHOR:      Jeremy Greene
 * COMPANY:     BitFlow, Inc.
 * COPYRIGHT:   Copyright (C) 2015, BitFlow, Inc.
 * DESCRIPTION: Implement a Qt class providing a popover style window, for trivial windows.
 */

#include <QFrame>

namespace ImageView
{
    class PopoverWindow : public QFrame
    {
        Q_OBJECT;

        Q_PROPERTY(Qt::Orientation  layoutOrientation       READ layoutOrientation      WRITE setLayoutOrientation);

        Q_PROPERTY(bool             rejectButtonVisible     READ rejectButtonVisible    WRITE setRejectButtonVisible);
        Q_PROPERTY(bool             acceptButtonVisible     READ acceptButtonVisible    WRITE setAcceptButtonVisible);
                                                                                              
        Q_PROPERTY(bool             rejectOnFocusOut        READ rejectOnFocusOut       WRITE setRejectOnFocusOut);

    public:
        enum AffinityEnum
        {
            TopLeftAffinity,
            TopRightAffinity,
            BottomLeftAffinity,
            BottomRightAffinity,
            TopLeftOverAffinity,
            TopRightOverAffinity,
            BottomLeftOverAffinity,
            BottomRightOverAffinity
        };

    private:
        struct PrivateData;
        PrivateData &m_pd;

        // Illegal.
        PopoverWindow (void);
        PopoverWindow (PopoverWindow const&);
        PopoverWindow& operator= (PopoverWindow const&);

    public:
        PopoverWindow (QWidget *const parent = nullptr);
        ~PopoverWindow (void);

        QString labelText (void) const;
        void setLabelText (QString const& text);

        const QPixmap* labelPixmap (void) const;
        void setLabelPixmap (QPixmap const& pixmap);

        QWidget* affinityWidget (void) const;
        void setAffinityWidget (QWidget *const widget, const AffinityEnum affinity = BottomLeftAffinity);

        QWidget* centralWidget (void) const;
        void setCentralWidget (QWidget *const widget);

        // Properties.
        Qt::Orientation layoutOrientation (void) const;
        void setLayoutOrientation (const Qt::Orientation orientation);

        bool rejectButtonVisible (void) const;
        void setRejectButtonVisible (const bool visible);

        bool acceptButtonVisible (void) const;
        void setAcceptButtonVisible (const bool visible);

        bool rejectOnFocusOut (void) const;
        void setRejectOnFocusOut (const bool enabled);

    public slots:
        void accept (void);
        void reject (void);

        void open (void);

    signals:
        void accepted (void);
        void rejected (void);

        void opened (void);
        void closed (void);

    private:
        // Event overrides.
        void showEvent (QShowEvent *) override;
        void focusOutEvent (QFocusEvent *const evt) override;
        void closeEvent (QCloseEvent *const evt) override;
    };
}

#endif // INCLUDED__BIT_FLOW__IMAGE__VIEW__POPOVER__H
