#ifndef INCLUDED__BIT_FLOW__IMAGE__VIEW__SURFACE__SCROLL__AREA__H
#define INCLUDED__BIT_FLOW__IMAGE__VIEW__SURFACE__SCROLL__AREA__H

/* FILE:        ImageViewSurfaceScrollArea.h
 * DATE:        1/7/2016
 * AUTHOR:      Jeremy Greene
 * COMPANY:     BitFlow, Inc.
 * COPYRIGHT:   Copyright (C) 2016, BitFlow, Inc.
 * DESCRIPTION: Container widget for ImageView::Surface, providing scroll bars
 *              and an optional righthand-bottom corner widget.
 */

#include "ImageView.h"

#include <QWidget>
#include <QScrollBar>

namespace ImageView
{
    class IMAGEVIEW_EXPORT SurfaceScrollArea : public QWidget
    {
        Q_OBJECT

        Q_PROPERTY(double   zoomScaling     READ zoomScaling    WRITE setZoomTo);
        Q_PROPERTY(bool     zoomToFit       READ zoomToFit      WRITE setZoomToFit);
        Q_PROPERTY(QPoint   imageOffset     READ imageOffset    WRITE setImageOffset);

        Q_PROPERTY(bool     hBarScrollable  READ hBarScrollable);
        Q_PROPERTY(bool     vBarScrollable  READ vBarScrollable);

    private:
        struct PrivateData;
        PrivateData &m_pd;

    public:
        SurfaceScrollArea (QWidget *const parent);
        ~SurfaceScrollArea (void);

        // ImageView::Surface accessor.
        Surface* surface (void);
        const Surface* surface (void) const;

        // Scrollbar accessors.
        QScrollBar* horizontalScrollBar (void);
        const QScrollBar* horizontalScrollBar (void) const;

        QScrollBar* verticalScrollBar (void);
        const QScrollBar* verticalScrollBar (void) const;

        // Corner widget. The ImageView::Area takes ownership of the corner widget.
        QWidget* cornerWidget (void);
        const QWidget* cornerWidget (void) const;
        void setCornerWidget (QWidget *const widget);

        // Property accessors.
        double zoomScaling (void) const;
        bool zoomToFit (void) const;
        QPoint imageOffset (void) const;

        bool hBarScrollable (void) const;
        bool vBarScrollable (void) const;

    public slots:
        void setZoomTo (const double scale, QPoint const& centerRef = QPoint(-1, -1));
        void doZoomIn (QPoint const& centerRef = QPoint(-1, -1));
        void doZoomOut (QPoint const& centerRef = QPoint(-1, -1));
        void setZoomToFit (const bool enabled = true);

        void doZoomTo25 (void);
        void doZoomTo50 (void);
        void doZoomTo100 (void);
        void doZoomTo200 (void);
        void doZoomTo400 (void);
        void doZoomTo800 (void);

        void stepLeft (void);
        void stepRight (void);
        void stepUp (void);
        void stepDown (void);

        void pageLeft (void);
        void pageRight (void);
        void pageUp (void);
        void pageDown (void);

        void setImageOffset (QPointF const& offset);

    signals:
        void scrollabilityChanged (bool hBar, bool vBar);
    };
}

#endif // INCLUDED__BIT_FLOW__IMAGE__VIEW__SURFACE__SCROLL__AREA__H
