#ifndef INCLUDED__BIT_FLOW__IMAGE__VIEW__AREA_____PRIVATE__H
#define INCLUDED__BIT_FLOW__IMAGE__VIEW__AREA_____PRIVATE__H

/* FILE:        ImageViewSurfaceScrollArea_Private.h
 * DATE:        1/7/2016
 * AUTHOR:      Jeremy Greene
 * COMPANY:     BitFlow, Inc.
 * COPYRIGHT:   Copyright (C) 2016, BitFlow, Inc.
 * DESCRIPTION: PrivateData for ImageView::SurfaceScrollArea.
 */

#include "ImageViewSurfaceScrollArea.h"
#include "ImageViewSurface.h"

#include <QGridLayout>
#include <QShortcut>

#include <QEvent>
#include <QResizeEvent>
#include <QWheelEvent>
#include <QMouseEvent>
#include <QEnterEvent>

struct ImageView::SurfaceScrollArea::PrivateData : public QObject
{
    Q_OBJECT

public:
    PrivateData (SurfaceScrollArea &scrollArea);

    SurfaceScrollArea &m_scrollArea;

    QGridLayout *m_gridLayout;

    Surface *m_imageSurface;

    QScrollBar *m_horizontalScrollBar;
    QScrollBar *m_verticalScrollBar;

    QWidget *m_cornerWidget;

    QShortcut *m_leftShortcut, *m_rightShortcut, *m_upShortcut, *m_downShortcut;
    QShortcut *m_ctrlLeftShortcut, *m_ctrlRightShortcut, *m_ctrlUpShortcut, *m_ctrlDownShortcut;

public slots:
    void updateScrollRange (void);
    void updateScrollPos (QPointF const& offset);
};

class ViewportEventFilter : public QObject
{
    Q_OBJECT

private:
    // Members.
    ImageView::SurfaceScrollArea *const m_area;
    ImageView::Surface *const m_surface;

public:
    ViewportEventFilter (ImageView::SurfaceScrollArea *const area, ImageView::Surface *const surface);

private:
    virtual bool eventFilter (QObject *o, QEvent *e);

    bool wheelEvent (QObject *o, QWheelEvent *evt);
};

class ImageSurfaceEventFilter : public QObject
{
    Q_OBJECT

private:
    // Members.
    ImageView::SurfaceScrollArea *const m_area;
    ImageView::Surface *const m_surface;

    bool m_dragActive;
    QPoint m_pressPos;
    QPoint m_dragPointInit;

public:
    ImageSurfaceEventFilter (ImageView::SurfaceScrollArea *const area, ImageView::Surface *const parent);

private:
    virtual bool eventFilter (QObject *o, QEvent *e);

    bool mousePressEvent (QObject *o, QMouseEvent *evt);
    bool mouseMoveEvent (QObject *o, QMouseEvent *evt);
    bool mouseReleaseEvent (QObject *o, QMouseEvent *evt);
    bool enterEvent (QObject *o, QEnterEvent *evt);
    bool leaveEvent (QObject *o, QEvent *evt);
};

#endif // INCLUDED__BIT_FLOW__IMAGE__VIEW__AREA_____PRIVATE__H
