#ifndef INCLUDED__IMAGE__SURFACE_____PRIVATE__H
#define INCLUDED__IMAGE__SURFACE_____PRIVATE__H

/* FILE:        ImageViewSurface_Private.h
 * DATE:        1/7/2016
 * AUTHOR:      Jeremy Greene
 * COMPANY:     BitFlow, Inc.
 * COPYRIGHT:   Copyright (C) 2016, BitFlow, Inc.
 * DESCRIPTION: PrivateData for ImageView::Surface.
 */

#include "ImageViewSurface.h"
#include "AbstractAcquisitionInterface.h"
#include "ImageViewImageAccessor.h"

#include "ImageViewThread.h"

#include <functional>

#include <condition_variable>
#include <atomic>

struct ImageView::Surface::PrivateData
{
    PrivateData (ImageView::Surface &parent);
    ~PrivateData (void);

    //// Statics.
    static const double s_paddingRatio;

    //// Members.
    ImageView::Surface &m_parent;

    QPixmap m_pixmap;
    mutable std::mutex m_pixmapMutex;

#if IMAGE_VIEW_SURFACE_USE_QGL
    uint m_pixmapTexture;
    std::atomic<bool> m_textureExpired;
#endif

    AbstractAcquisitionInterface *m_acqIf;
    RawImage m_rawImage;

    QFont m_painterFont;
    const int m_painterFontPointSizeDefault;

    QImage m_image;

    QSizeF m_imagePixPosSizeMax;
    QSizeF m_imagePixValSizeMax;
    QSizeF m_overallPixSizeMax;
    std::atomic<bool> m_imageBufferSizeChange;

    mutable std::mutex m_imageBufferMutex;

    double m_scaling;
    QPointF m_offset;
    bool m_scaleToFit;

    bool m_crosshairVisible;
    QPoint m_activePixel;

    bool m_valueOverlayEnabled;

    ImageAccessor m_imageAccessor;

    //// Helpers.

    bool renderPixmap (QPainter &painter, QRectF const& targetRect);
    bool renderCrosshair (QPainter &painter, QRectF const& targetRect);
    bool renderPixelOverlay (QPainter &painter, QRectF const& targetRect);

    typedef QVector< QPair<QPointF, PixVal> > PixPairVector;
    bool unlocked_overlayPixelsFromBuffer (std::lock_guard<std::mutex> const& lock, QRectF const& targetRect, PixPairVector &pixPairs, QSizeF &posSizeMax, QSizeF &valSizeMax);
    bool unlocked_overlayPixelsFromImage (std::lock_guard<std::mutex> const& lock, QRectF const& targetRect, PixPairVector &pixPairs, QSizeF &posSizeMax, QSizeF &valSizeMax);
    void unlocked_findOptimalOverlaySizes (std::lock_guard<std::mutex> const& lock, QString const& posStr, QString const& valStr);

    // Methods accessing locked features that require external locking of m_pixmapMutex.
    void unlocked_setFittedOffset (std::lock_guard<std::mutex> const& lock, QPointF const& offset);
    bool unlocked_checkScaleToFit (std::lock_guard<std::mutex> const& lock, const bool setEnabled = false);

    QSizeF unlocked_scaledSize (std::lock_guard<std::mutex> const& lock) const;
    QRectF unlocked_scaledRect (std::lock_guard<std::mutex> const& lock) const;
    QPointF unlocked_scaledOffset (std::lock_guard<std::mutex> const& lock) const;
    QRectF unlocked_visibleImageRectF (std::lock_guard<std::mutex> const& lock) const;
};

#endif // INCLUDED__IMAGE__SURFACE_____PRIVATE__H
